@@ -17,12 +17,14 @@ import asyncio
 import typing
 import decimal
 import time
+import contextlib
 
 import octobot_commons.constants
 import octobot_commons.enums as common_enums
 import octobot_commons.logging as logging
 import octobot_commons.timestamp_util as timestamp_util
 import octobot_commons.tentacles_management as tentacles_management
+import octobot_commons.html_util as html_util
 
 import octobot_trading.constants
 import octobot_trading.enums as enums
@@ -71,7 +73,7 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
     }
     ACCOUNTS = {}
 
-    def __init__(self, config, exchange_manager):
+    def __init__(self, config, exchange_manager, exchange_config_by_exchange: typing.Optional[dict[str, dict]]):
         super().__init__()
         self.config = config
         self.exchange_manager = exchange_manager
@@ -94,6 +96,10 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
 
         self.is_unreachable = False
 
+        self._creating_exchange_order_descriptions = set()
+
+        if exchange_config_by_exchange and self.get_name() in exchange_config_by_exchange:
+            self.tentacle_config = exchange_config_by_exchange[self.get_name()]
         if self.exchange_manager.tentacles_setup_config is not None:
             self.load_user_inputs_from_class(self.exchange_manager.tentacles_setup_config, self.tentacle_config)
 
@@ -295,6 +301,17 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         """
         raise NotImplementedError("get_closed_orders is not implemented")
 
+    async def get_cancelled_orders(self, symbol: str = None, since: int = None,
+                                   limit: int = None, **kwargs: dict) -> list:
+        """
+        Get the user closed order list
+        :param symbol: the order symbol
+        :param since: the starting timestamp
+        :param limit: the list limit size
+        :return: the user closed order list
+        """
+        raise NotImplementedError("get_cancelled_orders is not implemented")
+
     async def get_my_recent_trades(self, symbol: str = None, since: int = None,
                                    limit: int = None, **kwargs: dict) -> list:
         """
@@ -313,6 +330,13 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         """
         raise NotImplementedError("get_leverage_tiers is not implemented")
 
+    async def cancel_all_orders(self, symbol: str = None, **kwargs: dict) -> None:
+        """
+        Cancel all orders on the exchange
+        :param symbol: the orders symbol
+        """
+        raise NotImplementedError("cancel_all_orders is not implemented")
+
     async def cancel_order(
             self, exchange_order_id: str, symbol: str, order_type: enums.TraderOrderType, **kwargs: dict
     ) -> enums.OrderStatus:
@@ -380,7 +404,7 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         """
         raise NotImplementedError("get_bundled_order_parameters is not implemented")
 
-    def is_supported_order_type(self, order_type):
+    def is_supported_order_type(self, order_type: enums.TraderOrderType) -> bool:
         """
         Check if the order type is supported by the current exchange instance
         Should be used to know if we should simulate this order or create it on the exchange
@@ -389,6 +413,12 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         """
         return order_type not in self.get_supported_elements(enums.ExchangeSupportedElements.UNSUPPORTED_ORDERS)
 
+    def is_market_open_for_order_type(self, symbol: str, order_type: enums.TraderOrderType) -> bool:
+        """
+        Override if necessary
+        """
+        return True
+
     def get_trade_fee(self, symbol: str, order_type: enums.TraderOrderType, quantity, price, taker_or_maker):
         """
         Calculates fees resulting to a trade
@@ -471,6 +501,8 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         t0 = time.time()
         minimal_interval = 0.1
         attempt = 1
+        latest_error = None
+        latest_request_url = None
         while (timeout != 0 and time.time() - t0 < timeout) or (n_times != 0 and attempt <= n_times + 1):
             last_request_time = time.time()
             try:
@@ -479,12 +511,21 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
                     self.logger.debug(f"Request retrier success for {request_func.__name__} after {attempt} attempts")
                 return result
             except errors.FailedRequest as err:
-                self.logger.debug(f"Request retrier failed for {request_func.__name__}({args} {kwargs}) (attempt {attempt}) ({err})")
+                latest_error = err
+                latest_request_url = self.get_latest_request_url()
+                self.logger.debug(
+                    f"Request retrier failed for {request_func.__name__}({args} {kwargs}) "
+                    f"(attempt {attempt}) ({html_util.get_html_summary_if_relevant(err)})"
+                )
                 if time.time() - last_request_time < minimal_interval:
                     await asyncio.sleep(minimal_interval)
                 attempt += 1
-        raise errors.FailedRequest(f"Failed to successfully run {request_func.__name__} request after {attempt} "
-                                   f"attempts.")
+        latest_error = latest_error or RuntimeError("unknown error, this is unexpected, latest_error should be set")
+        raise errors.FailedRequest(
+            f"Failed to successfully run {request_func.__name__}(args={args}, kwargs={kwargs}) request after {attempt} "
+            f"attempts. Latest error: {latest_error} ({latest_error.__class__.__name__}). "
+            f"Last request url: {latest_request_url}"
+        ) from latest_error
 
     """
     Parsers
@@ -585,6 +626,12 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
         """
         raise NotImplementedError("parse_account is not implemented")
 
+    def get_latest_request_url(self) -> str:
+        """
+        :return: the URL of the last request
+        """
+        return self.connector.get_latest_request_url()
+
     """
     Uniformization
     """
@@ -677,8 +724,55 @@ class AbstractExchange(tentacles_management.AbstractTentacle):
     def log_order_creation_error(self, error, order_type, symbol, quantity, price, stop_price):
         order_desc = f"order_type: {order_type}, symbol: {symbol}, quantity: {str(quantity)}, price: {str(price)}," \
                      f" stop_price: {str(stop_price)}"
-        self.logger.error(f"Failed to create order : {error.__class__.__name__} {error}: ({order_desc})")
+        self.logger.error(
+            f"Failed to create order : {error.__class__.__name__} {html_util.get_html_summary_if_relevant(error)}: "
+            f"({order_desc})"
+        )
 
     def handle_token_error(self, error):
         self.logger.error(f"Exchange configuration is invalid : please check your configuration ! "
-                          f"({error.__class__.__name__}: {error})")
+                          f"({error.__class__.__name__}: {html_util.get_html_summary_if_relevant(error)})")
+
+    @contextlib.contextmanager
+    def creating_order(
+        self, side: enums.TradeOrderSide, symbol: str, quantity: decimal.Decimal, price: decimal.Decimal
+    ):
+        desc = self._get_order_description(side, symbol, quantity, price)
+        try:
+            self._creating_exchange_order_descriptions.add(desc)
+            yield
+        finally:
+            try:
+                self._creating_exchange_order_descriptions.remove(desc)
+            except KeyError:
+                self.logger.error(f"Failed to remove {desc} from exchange order descriptions")
+
+    def is_creating_order(
+        self, order: dict, symbol: str
+    ) -> bool:
+        try:
+            quantity = decimal.Decimal(str(order[enums.ExchangeConstantsOrderColumns.AMOUNT.value] or 0))
+        except decimal.DecimalException:
+            quantity = octobot_trading.constants.ZERO
+        try:
+            price = decimal.Decimal(str(
+                order[enums.ExchangeConstantsOrderColumns.PRICE.value]
+                # when fetching stop/tp orders PRICE might not be set, also check STOP_PRICE and TAKE_PROFIT_PRICE
+                or order.get(enums.ExchangeConstantsOrderColumns.STOP_PRICE.value, 0)
+                or order.get(enums.ExchangeConstantsOrderColumns.TAKE_PROFIT_PRICE.value, 0)
+                or 0
+            ))
+        except decimal.DecimalException:
+            price = octobot_trading.constants.ZERO
+        side = enums.TradeOrderSide(
+            str(order[enums.ExchangeConstantsOrderColumns.SIDE.value]) or enums.TradeOrderSide.BUY.value
+        )
+        return self._get_order_description(side, symbol, quantity, price) in self._creating_exchange_order_descriptions
+
+    def _get_order_description(
+        self, side: enums.TradeOrderSide, symbol: str, quantity: decimal.Decimal, price: decimal.Decimal
+    ) -> str:
+        return (
+            f"{side.value if side else None}-{symbol}-"
+            f"{float(quantity) if quantity else None}-{float(price) if price else None}"
+        )
