@@ -14,6 +14,7 @@
 #  You should have received a copy of the GNU Lesser General Public
 #  License along with this library.
 import decimal
+import time
 
 import octobot_trading.exchanges.adapters as adapters
 import octobot_trading.exchanges.connectors.ccxt.enums as ccxt_enums
@@ -24,6 +25,7 @@ import octobot_trading.enums as enums
 from octobot_trading.enums import ExchangeConstantsOrderColumns as ecoc
 import octobot_commons.enums as common_enums
 import octobot_commons.constants as common_constants
+import octobot_commons.number_util as number_util
 
 
 class CCXTAdapter(adapters.AbstractAdapter):
@@ -42,6 +44,30 @@ class CCXTAdapter(adapters.AbstractAdapter):
         # CCXT standard order parsing logic
         return fixed
 
+    def adapt_amount_from_filled_or_cost(self, fixed):
+        try:
+            if (
+                fixed[enums.ExchangeConstantsOrderColumns.TYPE.value] == enums.TradeOrderType.MARKET.value and
+                fixed[enums.ExchangeConstantsOrderColumns.SIDE.value] == enums.TradeOrderSide.BUY.value and
+                fixed[enums.ExchangeConstantsOrderColumns.FILLED.value]
+            ):
+                # convert amount to use the base unit: use FILLED for accuracy (when not None/0)
+                fixed[enums.ExchangeConstantsOrderColumns.AMOUNT.value] = (
+                    fixed[enums.ExchangeConstantsOrderColumns.FILLED.value]
+                )
+            if (
+                (not fixed[enums.ExchangeConstantsOrderColumns.AMOUNT.value]) and
+                fixed[enums.ExchangeConstantsOrderColumns.COST.value] and
+                fixed[enums.ExchangeConstantsOrderColumns.PRICE.value]
+            ):
+                # convert amount to use the base unit
+                fixed[enums.ExchangeConstantsOrderColumns.AMOUNT.value] = (
+                    fixed[enums.ExchangeConstantsOrderColumns.COST.value] /
+                    fixed[enums.ExchangeConstantsOrderColumns.PRICE.value]
+                )
+        except KeyError:
+            pass
+
     def adapt_quantities_with_contract_size(self, order_or_trade, symbol):
         if self.connector.exchange_manager.is_future:
             symbol = symbol or order_or_trade.get(ecoc.SYMBOL.value)
@@ -174,6 +200,17 @@ class CCXTAdapter(adapters.AbstractAdapter):
     def fix_order_book(self, raw, **kwargs):
         fixed = super().fix_order_book(raw, **kwargs)
         # CCXT standard order_book fixing logic
+        try:
+            exchange_timestamp = fixed[enums.ExchangeConstantsOrderBookInfoColumns.TIMESTAMP.value]
+            if exchange_timestamp is None:
+                # force current time
+                fixed[enums.ExchangeConstantsOrderBookInfoColumns.TIMESTAMP.value] =  time.time()
+            else:
+                fixed[enums.ExchangeConstantsOrderBookInfoColumns.TIMESTAMP.value] = self.get_uniformized_timestamp(
+                    exchange_timestamp
+                )
+        except KeyError as e:
+            self.logger.error(f"Fail to convert order book timestamp ({e})")
         return fixed
 
     def parse_order_book(self, fixed, **kwargs):
@@ -236,21 +273,36 @@ class CCXTAdapter(adapters.AbstractAdapter):
         # CCXT standard position parsing logic
         # if mode is enums.PositionMode.ONE_WAY:
         original_side = fixed.get(ccxt_enums.ExchangePositionCCXTColumns.SIDE.value)
-        position_side = enums.PositionSide.BOTH
-        # todo when handling cross positions
-        # side = fixed.get(ccxt_enums.ExchangePositionCCXTColumns.SIDE.value, enums.PositionSide.UNKNOWN.value)
-        # position_side = enums.PositionSide.LONG \
-        #     if side == enums.PositionSide.LONG.value else enums.PositionSide.
         symbol = fixed.get(ccxt_enums.ExchangePositionCCXTColumns.SYMBOL.value)
         contract_size = decimal.Decimal(str(fixed.get(ccxt_enums.ExchangePositionCCXTColumns.CONTRACT_SIZE.value, 0)))
         contracts = constants.ZERO if force_empty \
             else decimal.Decimal(str(fixed.get(ccxt_enums.ExchangePositionCCXTColumns.CONTRACTS.value, 0)))
         is_empty = contracts == constants.ZERO
+        position_mode = (
+            enums.PositionMode.HEDGE if fixed.get(ccxt_enums.ExchangePositionCCXTColumns.HEDGED.value, False)
+            else enums.PositionMode.ONE_WAY
+        )
+        if position_mode is enums.PositionMode.HEDGE:
+            # todo when handling hedge positions
+            side = fixed.get(ccxt_enums.ExchangePositionCCXTColumns.SIDE.value, enums.PositionSide.UNKNOWN.value)
+            position_side = enums.PositionSide.LONG \
+                if side == enums.PositionSide.LONG.value else enums.PositionSide.SHORT
+            log_func = self.logger.debug
+            if is_empty:
+                log_func = self.logger.warning
+            log_func(f"Unhandled {symbol} position mode ({position_mode.value}). This position can't be traded.")
+        else:
+            # One way position use BOTH side as there is always only one position per symbol.
+            # This position can turn long and short
+            position_side = enums.PositionSide.BOTH
         liquidation_price = fixed.get(ccxt_enums.ExchangePositionCCXTColumns.LIQUIDATION_PRICE.value, 0)
-        if margin_type := fixed.get(ccxt_enums.ExchangePositionCCXTColumns.MARGIN_TYPE.value, None):
+        if margin_type := fixed.get(
+            ccxt_enums.ExchangePositionCCXTColumns.MARGIN_TYPE.value,
+            fixed.get(ccxt_enums.ExchangePositionCCXTColumns.MARGIN_MODE.value, None)   # can also be contained in margin mode
+        ):
             margin_type = enums.MarginType(margin_type)
         if force_empty or liquidation_price is None:
-            liquidation_price = constants.NaN
+            liquidation_price = constants.ZERO
         else:
             liquidation_price = decimal.Decimal(str(liquidation_price))
         try:
@@ -269,29 +321,28 @@ class CCXTAdapter(adapters.AbstractAdapter):
                 enums.ExchangeConstantsPositionColumns.LEVERAGE.value:
                     self.safe_decimal(fixed, ccxt_enums.ExchangePositionCCXTColumns.LEVERAGE.value,
                                       constants.DEFAULT_SYMBOL_LEVERAGE),
-                enums.ExchangeConstantsPositionColumns.POSITION_MODE.value: None if is_empty else
-                enums.PositionMode.HEDGE if fixed.get(ccxt_enums.ExchangePositionCCXTColumns.HEDGED.value, True)
-                else enums.PositionMode.ONE_WAY,
+                enums.ExchangeConstantsPositionColumns.POSITION_MODE.value: position_mode,
                 # next values are always 0 when the position empty (0 contracts)
                 enums.ExchangeConstantsPositionColumns.COLLATERAL.value: constants.ZERO if is_empty else
                 decimal.Decimal(
-                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.COLLATERAL.value, 0)}"),
+                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.COLLATERAL.value, 0) or 0}"),
                 enums.ExchangeConstantsPositionColumns.NOTIONAL.value: constants.ZERO if is_empty else
                 decimal.Decimal(
-                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.NOTIONAL.value, 0)}"),
+                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.NOTIONAL.value, 0) or 0}"),
                 enums.ExchangeConstantsPositionColumns.INITIAL_MARGIN.value: constants.ZERO if is_empty else
                 decimal.Decimal(
-                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.INITIAL_MARGIN.value, 0)}"),
+                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.INITIAL_MARGIN.value, 0) or 0}"),
+                enums.ExchangeConstantsPositionColumns.AUTO_DEPOSIT_MARGIN.value: False,    # default value
                 enums.ExchangeConstantsPositionColumns.UNREALIZED_PNL.value: constants.ZERO if is_empty else
                 decimal.Decimal(
-                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.UNREALISED_PNL.value, 0)}"),
+                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.UNREALISED_PNL.value, 0) or 0}"),
                 enums.ExchangeConstantsPositionColumns.REALISED_PNL.value: constants.ZERO if is_empty else
                 decimal.Decimal(
                     f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.REALISED_PNL.value, 0) or 0}"),
                 enums.ExchangeConstantsPositionColumns.LIQUIDATION_PRICE.value: liquidation_price,
                 enums.ExchangeConstantsPositionColumns.MARK_PRICE.value: constants.ZERO if is_empty else
                 decimal.Decimal(
-                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.MARK_PRICE.value, 0)}"),
+                    f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.MARK_PRICE.value, 0) or 0}"),
                 enums.ExchangeConstantsPositionColumns.ENTRY_PRICE.value: constants.ZERO if is_empty else
                 decimal.Decimal(
                     f"{fixed.get(ccxt_enums.ExchangePositionCCXTColumns.ENTRY_PRICE.value, 0)}"),
@@ -334,11 +385,8 @@ class CCXTAdapter(adapters.AbstractAdapter):
         return fixed
 
     def parse_leverage(self, fixed, **kwargs):
-        # WARNING no CCXT standard leverage parsing logic
-        # HAS TO BE IMPLEMENTED IN EACH EXCHANGE IMPLEMENTATION
-        return {
-            enums.ExchangeConstantsLeveragePropertyColumns.RAW.value: fixed,
-        }
+        # CCXT standard leverage fixing logic
+        return fixed
 
     def fix_funding_rate_history(self, raw, **kwargs):
         fixed = super().fix_funding_rate_history(raw, **kwargs)
@@ -400,11 +448,31 @@ class CCXTAdapter(adapters.AbstractAdapter):
         # CCXT standard mark_price parsing logic
         return fixed
 
-    def fix_market_status(self, raw, **kwargs):
-        fixed = super().fix_market_status(raw, **kwargs)
+    def fix_market_status(self, raw, remove_price_limits=False, **kwargs):
+        fixed = super().fix_market_status(raw, remove_price_limits=remove_price_limits, **kwargs)
+        if not fixed:
+            return fixed
         # CCXT standard market_status fixing logic
+        fixed[enums.ExchangeConstantsMarketStatusColumns.PRECISION.value][
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_AMOUNT.value] = number_util.get_digits_count(
+            fixed[enums.ExchangeConstantsMarketStatusColumns.PRECISION.value][
+                enums.ExchangeConstantsMarketStatusColumns.PRECISION_AMOUNT.value]
+        )
+        fixed[enums.ExchangeConstantsMarketStatusColumns.PRECISION.value][
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_PRICE.value] = number_util.get_digits_count(
+            fixed[enums.ExchangeConstantsMarketStatusColumns.PRECISION.value][
+                enums.ExchangeConstantsMarketStatusColumns.PRECISION_PRICE.value]
+        )
+        if remove_price_limits:
+            fixed[enums.ExchangeConstantsMarketStatusColumns.LIMITS.value][
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE.value][
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MIN.value] = None
+            fixed[enums.ExchangeConstantsMarketStatusColumns.LIMITS.value][
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE.value][
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MAX.value] = None
+
         return fixed
 
-    def parse_market_status(self, fixed, **kwargs):
+    def parse_market_status(self, fixed, remove_price_limits=False, **kwargs):
         # CCXT standard market_status parsing logic
         return fixed
