@@ -1,3 +1,4 @@
+# pylint: disable=W0706
 #  Drakkar-Software OctoBot-Trading
 #  Copyright (c) Drakkar-Software, All rights reserved.
 #
@@ -18,6 +19,7 @@ import decimal
 import typing
 
 import octobot_commons.logging as logging
+import octobot_commons.html_util as html_util
 import octobot_commons.constants
 
 import octobot_trading.personal_data.orders.order_factory as order_factory
@@ -56,6 +58,7 @@ class Trader(util.Initializable):
         if not hasattr(self, 'simulate'):
             self.simulate = False
         self.is_enabled = self.__class__.enabled(self.config)
+        self.enable_inactive_orders = not self.simulate
 
     async def initialize_impl(self):
         self.is_enabled = self.is_enabled and self.exchange_manager.is_trading
@@ -70,6 +73,9 @@ class Trader(util.Initializable):
     def enabled(cls, config):
         return util.is_trader_enabled(config)
 
+    def set_enable_inactive_orders(self, enabled: bool):
+        self.enable_inactive_orders = enabled
+
     def set_risk(self, risk):
         min_risk = decimal.Decimal(str(octobot_commons.constants.CONFIG_TRADER_RISK_MIN))
         max_risk = decimal.Decimal(str(octobot_commons.constants.CONFIG_TRADER_RISK_MAX))
@@ -85,9 +91,10 @@ class Trader(util.Initializable):
     Orders
     """
 
-    async def create_order(self, order, loaded: bool = False, params: dict = None,
-                           wait_for_creation=True,
-                           creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT):
+    async def create_order(
+        self, order, loaded: bool = False, params: dict = None, wait_for_creation=True, raise_all_creation_error=False,
+        creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ):
         """
         Create a new order from an OrderFactory created order, update portfolio, registers order in order manager and
         notifies order channel.
@@ -97,6 +104,8 @@ class Trader(util.Initializable):
         :param wait_for_creation: when True, always make sure the order is completely created before returning.
         On exchanges async api, a create request will return before the order is actually live on exchange, in this case
         the associated order state will make sure that the order is creating by polling the order from the exchange.
+        :param raise_all_creation_error: when True, will raise each ceation error when possible
+        (instead of retuning None)
         :param creation_timeout: time before raising a timeout error when waiting for an order creation
         :return: The crated order instance
         """
@@ -111,37 +120,46 @@ class Trader(util.Initializable):
         try:
             params = params or {}
             self.logger.info(f"Creating order: {created_order}")
-            created_order = await self._create_new_order(order, params, wait_for_creation=wait_for_creation,
-                                                         creation_timeout=creation_timeout)
+            created_order = await self._create_new_order(order, params, wait_for_creation, creation_timeout)
             if created_order is None:
                 self.logger.warning(f"Order not created on {self.exchange_manager.exchange_name} "
                                     f"(failed attempt to create: {order}). This is likely due to "
                                     f"the order being refused by the exchange.")
-        except errors.MissingFunds:
+        except (
+            errors.MissingFunds, errors.AuthenticationError,
+            errors.ExchangeCompliancyError, errors.OrderCreationError
+        ):
             # forward errors that require actions to fix the situation
             raise
         except Exception as e:
+            if raise_all_creation_error:
+                raise
             self.logger.exception(e, True, f"Unexpected error when creating order: {e}. Order: {order}")
             return None
 
         return created_order
 
-    async def create_artificial_order(self, order_type, symbol, current_price, quantity, price,
-                                      emit_trading_signals=False,
-                                      wait_for_creation=True,
-                                      creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT):
+    async def create_artificial_order(
+        self, order_type, symbol, current_price, quantity, price, reduce_only, close_position,
+        emit_trading_signals=False, wait_for_creation=True,
+        creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ):
         """
         Creates an OctoBot managed order (managed orders example: stop loss that is not published on the exchange and
         that is maintained internally).
         """
-        order = order_factory.create_order_instance(trader=self,
-                                                    order_type=order_type,
-                                                    symbol=symbol,
-                                                    current_price=current_price,
-                                                    quantity=quantity,
-                                                    price=price)
+        order = order_factory.create_order_instance(
+            trader=self,
+            order_type=order_type,
+            symbol=symbol,
+            current_price=current_price,
+            quantity=quantity,
+            price=price,
+            reduce_only=reduce_only,
+            close_position=close_position,
+        )
         async with signals.remote_signal_publisher(self.exchange_manager, order.symbol, emit_trading_signals):
-            await signals.create_order(
+            return await signals.create_order(
                 self.exchange_manager,
                 emit_trading_signals and signals.should_emit_trading_signal(self.exchange_manager),
                 order,
@@ -168,47 +186,34 @@ class Trader(util.Initializable):
         previous_exchange_order_id = order.exchange_order_id
         try:
             async with order.lock:
+                disabled_state_updater = self.exchange_manager.exchange_personal_data \
+                    .orders_manager.enable_order_auto_synchronization is False
                 # now that we got the lock, ensure we can edit the order
-                if not self.simulate and not order.is_self_managed() and order.state is not None:
-                    # careful here: make sure we are not editing an order on exchange that is being updated
-                    # somewhere else
-                    async with order.state.refresh_operation():
-                        self.logger.info(f"Editing order: {order} ["
-                                         f"edited_quantity: {str(edited_quantity)} "
-                                         f"edited_price: {str(edited_price)} "
-                                         f"edited_stop_price: {str(edited_stop_price)} "
-                                         f"edited_current_price: {str(edited_current_price)}"
-                                         f"]")
-                        order_params = self.exchange_manager.exchange.get_order_additional_params(order)
-                        order_params.update(params or {})
-                        # fill in every param as some exchange rely on re-creating the order altogether
-                        edited_order = await self.exchange_manager.exchange.edit_order(
-                            order.exchange_order_id,
-                            order.order_type,
-                            order.symbol,
-                            quantity=order.origin_quantity if edited_quantity is None else edited_quantity,
-                            price=order.origin_price if edited_price is None else edited_price,
-                            stop_price=edited_stop_price,
-                            side=order.side,
-                            current_price=edited_current_price,
-                            params=order_params
+                if not self.simulate and order.is_active and not order.is_self_managed() and (
+                    order.state is not None or disabled_state_updater
+                ):
+                    if disabled_state_updater:
+                        changed = await self._edit_order_on_exchange(
+                            order, edited_quantity, edited_price, edited_stop_price, edited_current_price, params
                         )
-                        # apply new values from returned order (even order id might have changed)
-                        self.logger.debug(f"Successfully edited order on {self.exchange_manager.exchange_name}, "
-                                          f"new order values: {edited_order}")
-                        changed = order.update_from_raw(edited_order)
-                        # update portfolio from exchange
-                        await self.exchange_manager.exchange_personal_data.\
-                            handle_portfolio_and_position_update_from_order(order)
+                    else:
+                        # careful here: make sure we are not editing an order on exchange that is being updated
+                        # somewhere else
+                        async with order.state.refresh_operation():
+                            changed = await self._edit_order_on_exchange(
+                                order, edited_quantity, edited_price, edited_stop_price, edited_current_price, params
+                            )
                 else:
                     # consider order as cancelled to release portfolio amounts
                     self.exchange_manager.exchange_personal_data.portfolio_manager.portfolio.update_portfolio_available(
                         order, is_new_order=False
                     )
+                    updated_price = edited_stop_price if edited_price is None else edited_price
                     changed = order.update(
                         order.symbol,
                         quantity=edited_quantity,
-                        price=edited_stop_price if edited_price is None else edited_price,
+                        price=updated_price,
+                        filled_price=updated_price,
                         stop_price=edited_stop_price,
                         current_price=edited_current_price,
                     )
@@ -227,16 +232,53 @@ class Trader(util.Initializable):
                 # order id changed: update orders_manager to keep consistency
                 self.exchange_manager.exchange_personal_data.orders_manager.replace_order(order.order_id, order)
 
-    async def _create_new_order(self, new_order, params: dict,
-                                wait_for_creation=True,
-                                creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT) -> object:
+    async def _edit_order_on_exchange(
+        self, order,
+        edited_quantity: decimal.Decimal,
+        edited_price: decimal.Decimal,
+        edited_stop_price: decimal.Decimal,
+        edited_current_price: decimal.Decimal,
+        params: dict
+    ) -> bool:
+        self.logger.info(
+            f"Editing order: {order} [edited_quantity: {str(edited_quantity)} edited_price: {str(edited_price)} "
+            f"edited_stop_price: {str(edited_stop_price)} edited_current_price: {str(edited_current_price)}]"
+        )
+        order_params = self.exchange_manager.exchange.get_order_additional_params(order)
+        order_params.update(params or {})
+        # fill in every param as some exchange rely on re-creating the order altogether
+        edited_order = await self.exchange_manager.exchange.edit_order(
+            order.exchange_order_id,
+            order.order_type,
+            order.symbol,
+            quantity=order.origin_quantity if edited_quantity is None else edited_quantity,
+            price=order.origin_price if edited_price is None else edited_price,
+            stop_price=edited_stop_price,
+            side=order.side,
+            current_price=edited_current_price,
+            params=order_params
+        )
+        # apply new values from returned order (even order id might have changed)
+        self.logger.debug(
+            f"Successfully edited order on {self.exchange_manager.exchange_name}, new order values: {edited_order}"
+        )
+        changed = order.update_from_raw(edited_order)
+        # update portfolio from exchange
+        await self.exchange_manager.exchange_personal_data.handle_portfolio_and_position_update_from_order(order)
+        return changed
+
+    async def _create_new_order(
+        self, new_order, params: dict, wait_for_creation: bool, creation_timeout: float
+    ):
         """
         Creates an exchange managed order, it might be a simulated or a real order.
         Portfolio will be updated by the created order state after order will be initialized
         """
         updated_order = new_order
         is_pending_creation = False
-        if not self.simulate and not new_order.is_self_managed():
+        if not self.simulate and not new_order.is_self_managed() and (
+            new_order.is_in_active_inactive_transition or new_order.is_active
+        ):
             order_params = self.exchange_manager.exchange.get_order_additional_params(new_order)
             order_params.update(new_order.exchange_creation_params)
             order_params.update(params)
@@ -275,15 +317,32 @@ class Trader(util.Initializable):
             updated_order.is_waiting_for_chained_trigger = new_order.is_waiting_for_chained_trigger
             updated_order.associated_entry_ids = new_order.associated_entry_ids
             updated_order.update_with_triggering_order_fees = new_order.update_with_triggering_order_fees
+            updated_order.trailing_profile = new_order.trailing_profile
+            if new_order.active_trigger is not None:
+                updated_order.use_active_trigger(order_util.create_order_price_trigger(
+                    updated_order, new_order.active_trigger.trigger_price, new_order.active_trigger.trigger_above
+                ))
+            updated_order.is_in_active_inactive_transition = new_order.is_in_active_inactive_transition
 
             if is_pending_creation:
                 # register order as pending order, it will then be added to live orders in order manager once open
-                self.exchange_manager.exchange_personal_data.orders_manager.register_pending_creation_order(updated_order)
+                self.exchange_manager.exchange_personal_data.orders_manager.register_pending_creation_order(
+                    updated_order
+                )
 
-        await updated_order.initialize()
-        if is_pending_creation and wait_for_creation \
-                and updated_order.state is not None and updated_order.state.is_pending():
-            await updated_order.state.wait_for_terminate(creation_timeout)
+        try:
+            await updated_order.initialize()
+            if is_pending_creation and wait_for_creation \
+                    and updated_order.state is not None and updated_order.state.is_pending()\
+                    and self.exchange_manager.exchange_personal_data.orders_manager.enable_order_auto_synchronization:
+                await updated_order.state.wait_for_terminate(creation_timeout)
+            if new_order.is_in_active_inactive_transition:
+                # transition successful: new_order is now inactive
+                await new_order.on_active_from_inactive()
+        finally:
+            if updated_order.is_in_active_inactive_transition:
+                # transition completed: never leave is_in_active_inactive_transition to True after transition
+                updated_order.is_in_active_inactive_transition = False
         return updated_order
 
     def get_take_profit_order_type(self, base_order, order_type: enums.TraderOrderType) -> enums.TraderOrderType:
@@ -317,6 +376,7 @@ class Trader(util.Initializable):
             order, chained_order.order_type
         )
         if is_bundled:
+            # warning: doesn't work for multiple stop loss / take profits
             if chained_order.order_type is enums.TraderOrderType.STOP_LOSS:
                 params.update(self.exchange_manager.exchange.get_bundled_order_parameters(
                     order,
@@ -343,6 +403,36 @@ class Trader(util.Initializable):
         order.add_chained_order(chained_order)
         self.logger.info(f"Added chained order [{chained_order}] to [{order}] order.")
 
+    async def update_order_as_inactive(
+        self, order, ignored_order=None, wait_for_cancelling=True,
+        cancelling_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ) -> bool:
+        if not self.enable_inactive_orders:
+            self.logger.error(f"Can't update order as inactive when {self.enable_inactive_orders=}.")
+            return False
+        cancelled = False
+        if order and order.is_open():
+            with order.active_or_inactive_transition():
+                cancelled = await self._handle_order_cancellation(
+                    order, ignored_order, wait_for_cancelling, cancelling_timeout
+                )
+        else:
+            self.logger.error(f"Can't update order as inactive: {order} is not open on exchange.")
+        return cancelled
+
+    async def update_order_as_active(
+        self, order, params: dict = None, wait_for_creation=True, raise_all_creation_error=False,
+        creation_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ):
+        if not self.enable_inactive_orders:
+            self.logger.error(f"Can't update order as active when {self.enable_inactive_orders=}.")
+            return order
+        with order.active_or_inactive_transition():
+            return await self.create_order(
+                order, loaded=False, params=params, wait_for_creation=wait_for_creation,
+                raise_all_creation_error=raise_all_creation_error, creation_timeout=creation_timeout
+            )
+
     async def cancel_order(self, order, ignored_order=None,
                            wait_for_cancelling=True,
                            cancelling_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT) -> bool:
@@ -358,22 +448,26 @@ class Trader(util.Initializable):
         :param cancelling_timeout: time before raising a timeout error when waiting for an order cancel
         :return: None
         """
-        if order and order.is_open():
+        if order and order.is_open() or not order.is_active:
             self.logger.info(f"Cancelling order: {order}")
             # always cancel this order first to avoid infinite loop followed by deadlock
-            return await self._handle_order_cancellation(order, ignored_order,
-                                                         wait_for_cancelling, cancelling_timeout)
+            return await self._handle_order_cancellation(
+                order, ignored_order, wait_for_cancelling, cancelling_timeout
+            )
         return False
 
-    async def _handle_order_cancellation(self, order, ignored_order,
-                                         wait_for_cancelling: bool, cancelling_timeout: float) -> bool:
+    async def _handle_order_cancellation(
+        self, order, ignored_order, wait_for_cancelling: bool, cancelling_timeout: float
+    ) -> bool:
         success = True
         if order.is_waiting_for_chained_trigger:
             # order will just never get created
             order.is_waiting_for_chained_trigger = False
             return success
+        order_status = None
+        is_order_refreshing = False
         # if real order: cancel on exchange
-        if not self.simulate and not order.is_self_managed():
+        if not self.simulate and order.is_active and not order.is_self_managed():
             try:
                 async with order.lock:
                     try:
@@ -382,18 +476,28 @@ class Trader(util.Initializable):
                         )
                     except errors.NotSupported:
                         raise
-                    except (errors.OrderCancelError, Exception) as err:
+                    except (errors.OrderCancelError, Exception) as inner_err:
                         # retry to cancel order
-                        self.logger.debug(f"Failed to cancel order ({err} {err.__class__.__name__}), retrying")
+                        self.logger.info(
+                            f"Failed to cancel order ({inner_err} {inner_err.__class__.__name__}), retrying"
+                        )
                         order_status = await self.exchange_manager.exchange.cancel_order(
                             order.exchange_order_id, order.symbol, order.order_type
                         )
             except errors.OrderCancelError as err:
-                if await self._handle_order_cancel_error(order, err, wait_for_cancelling, cancelling_timeout):
-                    return True
-            except Exception as e:
-                self.logger.exception(e, True, f"Failed to cancel order {order}")
+                if self.exchange_manager.exchange_personal_data.orders_manager.enable_order_auto_synchronization:
+                    if await self._handle_order_cancel_error(order, err, wait_for_cancelling, cancelling_timeout):
+                        return True
+                else:
+                    self.logger.warning(
+                        f"Impossible to cancel order ({err} {err.__class__.__name__}). "
+                        f"Considering order as cancelled {order}"
+                    )
+                    order_status = enums.OrderStatus.CANCELED
+            except Exception as err:
+                self.logger.exception(err, True, f"Failed to cancel order {order}")
                 return False
+            is_order_refreshing = order.is_refreshing()
             if order_status is enums.OrderStatus.CANCELED:
                 order.status = octobot_trading.enums.OrderStatus.CANCELED
                 self.logger.debug(f"Successfully cancelled order {order}")
@@ -403,11 +507,14 @@ class Trader(util.Initializable):
         else:
             order.status = octobot_trading.enums.OrderStatus.CANCELED
 
-        await order.on_cancel(force_cancel=order.status is octobot_trading.enums.OrderStatus.CANCELED,
-                              is_from_exchange_data=False,
-                              ignored_order=ignored_order)
-        if wait_for_cancelling and order.state is not None and order.state.is_pending():
-            await self._wait_for_order_cancel(order, cancelling_timeout)
+        if not is_order_refreshing:
+            # don't override state if order is already refreshing (most likely from open orders updater)
+            await order.on_cancel(force_cancel=order.status is octobot_trading.enums.OrderStatus.CANCELED,
+                                  is_from_exchange_data=False,
+                                  ignored_order=ignored_order)
+        if wait_for_cancelling and (order.is_refreshing() or order.is_pending()):
+            # Don't wait for new state to avoid potential deadlock. Will raise if cancel is not in process
+            self._ensure_probably_canceled_order(order, None)
         return True
 
     async def _handle_order_cancel_error(self, order, err, wait_for_cancelling, cancelling_timeout):
@@ -425,23 +532,34 @@ class Trader(util.Initializable):
                 f"it impossible to handle this the issue. Please report it if you see it. "
                 f"Order: {order}"
             ) from err
-        # trigger forced refresh to get an update of the order
         if order.state.is_refreshing():
-            await order.state.wait_for_next_state(cancelling_timeout)
+            # can't wait for the order state to fully refresh as it will require a portfolio lock which might
+            # be locked by this ask already (and create a deadlock)
+            # => be optimistic and consider refreshing state as end state
+            if self._ensure_probably_canceled_order(order, err):
+                return True
         else:
+            # trigger forced refresh to get an update of the order
             previous_status = order.status
             await order.state.synchronize(force_synchronization=True)
             if previous_status != order.status:
-                # status changed: wait for state change
-                await order.state.wait_for_next_state(cancelling_timeout)
+                # don't wait for new state to avoid potential deadlock
+                if self._ensure_probably_canceled_order(order, err):
+                    return True
         if order.is_cancelled():
             self.logger.debug(f"Tried to cancel an already cancelled order. Order: {order}")
             return True
         if order.is_cancelling():
-            if wait_for_cancelling:
-                await self._wait_for_order_cancel(order, cancelling_timeout)
+            # don't wait for new state to avoid potential deadlock, just check that order is not filling
+            self._ensure_probably_canceled_order(order, err)
             return True
         elif order.is_open():
+            if isinstance(err, errors.OrderNotFoundOnCancelError):
+                raise errors.OrderNotFoundOnCancelError(
+                    f"Tried to cancel an order that can't be found, it might be cancelled or filled already "
+                    f"({html_util.get_html_summary_if_relevant(err)}). "
+                    f"Order: {order}"
+                ) from err
             raise errors.OpenOrderError(
                 f"Order is open, but can't be cancelled. This is unexpected. Order: {order}"
             ) from err
@@ -455,6 +573,72 @@ class Trader(util.Initializable):
                 f"Can't cancel order and unknown post sync order state for order: {order}."
             ) from err
 
+    def _ensure_probably_canceled_order(self, order, err: typing.Optional[Exception]) -> bool:
+        if order.is_refreshing_filling_state():
+            filled_err = errors.FilledOrderError(f"Order is filled, it can't be cancelled. Order: {order}")
+            if err is None:
+                raise filled_err
+            raise filled_err from err
+        if order.is_refreshing_canceling_state():
+            self.logger.debug(f"Tried to cancel an already cancelled order. Order: {order}")
+            return True
+        if order.is_pending_cancel_state():
+            self.logger.debug(f"Tried to cancel a pending cancel order. Order: {order}")
+            return True
+        return False
+
+    async def cancel_all_orders(
+        self,
+        symbol: str,
+        allow_single_order_cancel_fallback: bool,
+        wait_for_cancelling: bool = True,
+        cancelling_timeout: float = octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ) -> bool:
+        orders_to_cancel = self.exchange_manager.exchange_personal_data.orders_manager.get_all_orders(symbol)
+        success = True
+        if orders_to_cancel:
+            self.logger.info(f"Cancelling all {len(orders_to_cancel)} {symbol} orders")
+        else:
+            return success
+        try:
+            cancel_on_exchange = False
+            for order in orders_to_cancel:
+                await order.lock.acquire()
+                if self.simulate or order.is_self_managed():
+                    order.status = octobot_trading.enums.OrderStatus.CANCELED
+                else:
+                    cancel_on_exchange = True
+            if cancel_on_exchange:
+                try:
+                    success = False
+                    await self.exchange_manager.exchange.cancel_all_orders(symbol)
+                    success = True
+                except errors.NotSupported:
+                    if allow_single_order_cancel_fallback:
+                        self.logger.debug(
+                            f"cancel_all_orders is not supported on {self.exchange_manager.exchange_name}. Falling "
+                            f"back to one by one cancel"
+                        )
+                        for order in orders_to_cancel:
+                            if not await self.cancel_order(
+                                order.symbol,
+                                wait_for_cancelling=wait_for_cancelling,
+                                cancelling_timeout=cancelling_timeout
+                            ):
+                                success = False
+                    else:
+                        # not supported and no fallback allowed: re-raise errors.NotSupported
+                        raise
+            for order in orders_to_cancel:
+                await order.on_cancel(force_cancel=True, is_from_exchange_data=False)
+                if wait_for_cancelling and order.state is not None and order.state.is_pending():
+                    await self._wait_for_order_cancel(order, cancelling_timeout)
+        finally:
+            for order in orders_to_cancel:
+                order.lock.release()
+        self.logger.info(f"Cancelling of all {len(orders_to_cancel)} {symbol} orders complete")
+        return success
+
     async def _wait_for_order_cancel(self, order, cancelling_timeout):
         self.logger.debug(f"Waiting for order cancelling, order: {order}")
         await order.state.wait_for_terminate(cancelling_timeout)
@@ -525,9 +709,9 @@ class Trader(util.Initializable):
         return all_cancelled, cancelled_orders
 
     async def cancel_all_open_orders_with_currency(
-            self, currency, emit_trading_signals=False,
-            wait_for_cancelling=True,
-            cancelling_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+        self, currency, emit_trading_signals=False,
+        wait_for_cancelling=True,
+        cancelling_timeout=octobot_trading.constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
     ) -> bool:
         """
         Should be called only if the goal is to cancel all open orders for each traded symbol containing the
@@ -702,13 +886,16 @@ class Trader(util.Initializable):
         async with self.exchange_manager.exchange_personal_data.portfolio_manager.portfolio.lock:
             await self.exchange_manager.exchange_personal_data.handle_portfolio_update_from_withdrawal(amount, currency)
 
-    async def set_leverage(self, symbol, side, leverage):
+    async def set_leverage(
+        self, symbol: str, side: typing.Optional[enums.PositionSide], leverage: decimal.Decimal
+    ) -> bool:
         """
         Updates the symbol contract leverage
         Can raise InvalidLeverageValue if leverage value is not matching requirements
         :param symbol: the symbol to update
         :param side: the side to update (TODO)
         :param leverage: the new leverage value
+        :return True if leverage changed
         """
         contract = self.exchange_manager.exchange.get_pair_future_contract(symbol)
         if not contract.check_leverage_update(leverage):
@@ -716,12 +903,11 @@ class Trader(util.Initializable):
                                               f"but maximal value is {contract.maximum_leverage}")
         if contract.current_leverage != leverage:
             if not self.simulate:
-                await self.exchange_manager.exchange.set_symbol_leverage(
-                    symbol=symbol,
-                    leverage=leverage
-                )
+                await self.exchange_manager.exchange.set_symbol_leverage(symbol, float(leverage))
             self.logger.info(f"Switching {symbol} leverage from {contract.current_leverage} to {leverage}")
             contract.set_current_leverage(leverage)
+            return True
+        return False
 
     async def set_symbol_take_profit_stop_loss_mode(self, symbol, new_mode: enums.TakeProfitStopLossMode):
         """
