@@ -16,11 +16,14 @@
 import abc
 import contextlib
 import decimal
+import copy
+import typing
 
 import octobot_commons.constants as common_constants
 import octobot_commons.enums as common_enums
 import octobot_commons.logging as logging
 import octobot_commons.tentacles_management as abstract_tentacle
+import octobot_commons.configuration
 
 import async_channel.constants as channel_constants
 import async_channel.channels as channels
@@ -30,6 +33,7 @@ import octobot_tentacles_manager.configuration as tm_configuration
 
 import octobot_trading.constants as constants
 import octobot_trading.enums as enums
+import octobot_trading.errors as errors
 import octobot_trading.exchange_channel as exchanges_channel
 import octobot_trading.modes.modes_factory as modes_factory
 import octobot_trading.modes.channel.abstract_mode_producer as abstract_mode_producer
@@ -71,9 +75,15 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
         # The id of the OctoBot using this trading mode
         self.bot_id: str = None
 
-        # Evaluator specific config (Is loaded from tentacle specific file)
+        # Trading Mode specific config (Is loaded from tentacle specific file)
         self.trading_config: dict = None
 
+        # Trading Mode specific config snapshot before a config update
+        self.previous_trading_config: dict = None
+
+        # Used when historical config is enabled
+        self.historical_master_config: dict = None
+
         # If this mode is enabled
         self.enabled: bool = True
 
@@ -176,6 +186,13 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
         """
         return True
 
+    @classmethod
+    def is_ignoring_cancelled_orders_trades(cls) -> bool:
+        """
+        :return: True if trades created by cancelled orders should be ignored and not stored in trades manager
+        """
+        return False
+
     @classmethod
     def get_parent_trading_mode_classes(cls, higher_parent_class_limit=None) -> list:
         return [
@@ -192,6 +209,13 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
         """
         return True
 
+    def is_updating_exchange_settings(self, context) -> bool:
+        """
+        :return: True if the TradingMode should update exchange settings
+        (such as leverage) upon initializing user inputs
+        """
+        return True
+
     async def initialize(self, trading_config=None, auto_start=True) -> None:
         """
         Triggers producers and consumers creation
@@ -303,6 +327,10 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
             # when self.trading_config is None, should not happen
             return False
 
+    @classmethod
+    def get_forced_updater_channels(cls) -> set[str]:
+        return set()
+
     def _health_check_interval_expired(self) -> bool:
         return self.exchange_manager.exchange.get_exchange_current_time() - self._last_health_check_time \
             > self.HEALTH_CHECK_INTERVAL
@@ -346,7 +374,7 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
                 return created_orders
 
     async def single_exchange_process_optimize_initial_portfolio(
-        self, sellable_assets, target_asset: str, tickers: dict
+        self, sellable_assets: list, target_asset: str, tickers: dict
     ) -> list:
         raise NotImplementedError("single_exchange_process_optimize_initial_portfolio is not implemented")
 
@@ -418,6 +446,51 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
     def get_local_config(self):
         return self.trading_config
 
+    def update_config_and_user_inputs_if_necessary(self):
+        if self.update_config_if_necessary():
+            self.init_user_inputs({})
+
+    def update_config_if_necessary(self) -> bool:
+        if self.exchange_manager.is_backtesting:
+            try:
+                most_recent_config = self.get_historical_config()
+                if most_recent_config != self.trading_config:
+                    self.previous_trading_config = self.trading_config
+                    self.trading_config = most_recent_config
+                    self.logger.info(f"Switching config to use the most recent one: new config: {self.trading_config}")
+                    return True
+            except Exception as err:
+                self.logger.error(f"Error when reading historical config: {err} {err.__class__.__name__}")
+        return False
+
+    def get_historical_config(self) -> dict:
+        return octobot_commons.configuration.get_historical_tentacle_config(
+            self.get_historical_master_config(), self.exchange_manager.exchange.get_exchange_current_time()
+        )
+
+    def get_historical_configs(self, from_time: float, to_time: float) -> list[dict]:
+        try:
+            return octobot_commons.configuration.get_historical_tentacle_configs(
+                self.get_historical_master_config(), from_time, to_time
+            )
+        except KeyError:
+            return []
+
+    def get_historical_master_config(self):
+        if self.historical_master_config is None:
+            self.historical_master_config = copy.deepcopy(self.trading_config)
+        return self.historical_master_config
+
+    def supports_historical_config(self) -> bool:
+        return common_constants.CONFIG_HISTORICAL_CONFIGURATION in self.get_historical_master_config()
+
+    def has_historical_config(self) -> bool:
+        try:
+            octobot_commons.configuration.get_oldest_historical_tentacle_config_time(self.trading_config)
+            return True
+        except ValueError:
+            return False
+
     @classmethod
     def create_local_instance(cls, config, tentacles_setup_config, tentacle_config):
         return modes_factory.create_temporary_trading_mode_with_local_config(
@@ -428,14 +501,25 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
     def set_default_config(self) -> None:
         pass
 
+    def get_init_symbols(self) -> list:
+        try:
+            if self.has_historical_config():
+                self.update_config_and_user_inputs_if_necessary()
+            return self.get_tentacle_config_traded_symbols(
+                self.trading_config, self.exchange_manager.exchange_personal_data.portfolio_manager.reference_market
+            )
+        except NotImplementedError:
+            return self.exchange_manager.exchange_config.traded_symbol_pairs
+
+
     """
     Strategy related methods
     """
 
     @classmethod
-    def get_required_strategies_names_and_count(cls,
-                                                tentacles_config: tm_configuration.TentaclesSetupConfiguration,
-                                                trading_mode_config=None):
+    def get_required_strategies_names_and_count(
+        cls, tentacles_config: tm_configuration.TentaclesSetupConfiguration, trading_mode_config=None
+    ):
         config = trading_mode_config or tentacles_manager_api.get_tentacle_config(tentacles_config, cls)
         if constants.TRADING_MODE_REQUIRED_STRATEGIES in config:
             return config[constants.TRADING_MODE_REQUIRED_STRATEGIES], cls.get_required_strategies_count(config)
@@ -466,6 +550,13 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
             common_constants.DEFAULT_IGNORED_VALUE
         )
 
+    def ensure_supported(self, symbol):
+        if self.exchange_manager.is_future:
+            try:
+                self.exchange_manager.exchange.pair_contracts[symbol].ensure_supported_configuration()
+            except KeyError:
+                raise errors.ContractExistsError(f"Missing contract for {symbol}")
+
     @contextlib.asynccontextmanager
     async def remote_signal_publisher(self, symbol: str):
         async with signals.remote_signal_publisher(self.exchange_manager, symbol, self.should_emit_trading_signal()) \
@@ -481,10 +572,26 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
             wait_for_creation=wait_for_creation, creation_timeout=creation_timeout
         )
 
-    async def cancel_order(self, order, ignored_order: object = None) -> bool:
+    async def cancel_order(self, order, ignored_order: object = None, wait_for_cancelling: bool = True) -> bool:
         return await signals.cancel_order(
             self.exchange_manager, self.should_emit_trading_signal(), order,
-            ignored_order=ignored_order
+            ignored_order=ignored_order,
+            wait_for_cancelling=wait_for_cancelling
+        )
+
+    async def cancel_all_orders(
+        self,
+        symbol: str,
+        allow_single_order_cancel_fallback: bool,
+        wait_for_cancelling: bool = True,
+        cancelling_timeout: float = constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+    ) -> bool:
+        # todo send signals when implementation will be necessary
+        return await self.exchange_manager.trader.cancel_all_orders(
+            symbol,
+            allow_single_order_cancel_fallback,
+            wait_for_cancelling=wait_for_cancelling,
+            cancelling_timeout=cancelling_timeout
         )
 
     async def edit_order(self, order,
@@ -502,6 +609,13 @@ class AbstractTradingMode(abstract_tentacle.AbstractTentacle):
             params=params
         )
 
+    async def set_leverage(
+        self, symbol: str, side: typing.Optional[enums.PositionSide], leverage: decimal.Decimal
+    ) -> bool:
+        return await signals.set_leverage(
+            self.exchange_manager, self.should_emit_trading_signal(), symbol, side, leverage
+        )
+
     async def get_additional_metadata(self, is_backtesting):
         """
         Override if necessary
