@@ -16,6 +16,7 @@
 import contextlib
 import asyncio
 import concurrent.futures
+import typing
 
 import async_channel.enums as channel_enums
 
@@ -35,6 +36,7 @@ import octobot_trading.exchanges.exchanges as exchanges
 import octobot_trading.exchange_channel as exchanges_channel
 import octobot_trading.modes.channel as modes_channel
 import octobot_trading.modes.script_keywords as script_keywords
+import octobot_trading.modes.mode_activity as mode_activity
 import octobot_trading.storage.util as storage_util
 
 
@@ -47,7 +49,6 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
         common_enums.ActivationTopics.EVALUATION_CYCLE.value:
             channels_name.OctoBotEvaluatorsChannelsName.MATRIX_CHANNEL.value,
     }
-    CONFIG_INIT_TIMEOUT = 1 * common_constants.MINUTE_TO_SECONDS    # let time for orders to be fetched before
     # declaring timeout at first trigger
     PRODUCER_LOCKS_BY_EXCHANGE_ID = {}  # use to identify exchange-wide actions
 
@@ -95,6 +96,11 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
         self._is_trigger_completed = asyncio.Event()
         self._is_trigger_completed.set()
 
+        # used to avoid spamming critical issues
+        self._sent_critical_notifications: set[str] = set()
+
+        self.last_activity: mode_activity.TradingModeActivity = mode_activity.TradingModeActivity()
+
     def on_reload_config(self):
         """
         Called at constructor and after the associated trading mode's reload_config.
@@ -132,7 +138,8 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
             await self.inner_start()
         finally:
             self.logger.debug(
-                f"Ready to trade on {self.exchange_manager.exchange_name}, symbol: {self.trading_mode.symbol}"
+                f"Ready to trade on {self.exchange_manager.exchange_name if self.exchange_manager else ''}, "
+                f"symbol: {self.trading_mode.symbol if self.trading_mode.symbol else ''}"
             )
             self._is_ready_to_trade.set()
 
@@ -175,7 +182,19 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
                                                                          self.exchange_manager.id).matrix_id
             await self._subscribe_to_registration_topic(registration_topics, currency_filter, symbol_filter)
         await self.init_user_inputs(False)
-        await self._wait_for_bot_init(self.CONFIG_INIT_TIMEOUT)
+        await self._wait_for_bot_init(
+            self._get_config_init_timeout(),
+            extra_symbol_topics=self.get_extra_init_symbol_topics(),
+            extra_global_topics=self.get_extra_global_topics()
+        )
+
+    def get_extra_init_symbol_topics(self) -> typing.Optional[list]:
+        # Implement if necessary
+        return None
+
+    def get_extra_global_topics(self) -> typing.Optional[list]:
+        # Implement if necessary
+        return None
 
     async def _subscribe_to_registration_topic(self, registration_topics, currency_filter, symbol_filter):
         for registration_topic in registration_topics:
@@ -350,7 +369,7 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
                     raise asyncio.TimeoutError(f"Trading mode producer has to be started in backtesting")
                 self.logger.debug("Waiting for orders initialization to proceed")
                 try:
-                    await asyncio.wait_for(self._is_ready_to_trade.wait(), self.CONFIG_INIT_TIMEOUT)
+                    await asyncio.wait_for(self._is_ready_to_trade.wait(), self._get_config_init_timeout())
                 except asyncio.TimeoutError as e:
                     raise errors.InitializingError() from e
                 self.logger.debug("Order initialized")
@@ -401,21 +420,33 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
         """
         raise NotImplementedError("get_should_cancel_loaded_orders not implemented")
 
-    async def cancel_symbol_open_orders(self, symbol, side=None) -> bool:
+    async def cancel_symbol_open_orders(self, symbol, side=None, tag=None, exchange_order_ids=None) -> bool:
         """
         Cancel all symbol open orders
         """
         cancel_loaded_orders = self.get_should_cancel_loaded_orders()
         cancelled = False
+        failed_to_cancel = False
         if self.exchange_manager.trader.is_enabled:
-            for order in self.exchange_manager.exchange_personal_data.orders_manager.get_open_orders(symbol=symbol):
+            for order in self.exchange_manager.exchange_personal_data.orders_manager.get_open_orders(
+                symbol=symbol, tag=tag
+            ):
                 if (
                     not (order.is_cancelled() or order.is_closed())
                     and (cancel_loaded_orders or order.is_from_this_octobot)
                     and (side is None or (side is order.side))
+                    and (exchange_order_ids is None or (order.exchange_order_id in exchange_order_ids))
                 ):
-                    cancelled = await self.trading_mode.cancel_order(order) and cancelled
-        return cancelled
+                    try:
+                        if await self.trading_mode.cancel_order(order):
+                            cancelled = True
+                        else:
+                            failed_to_cancel = True
+                    except errors.OctoBotExchangeError as err:
+                        # do not propagate exchange error when canceling order
+                        self.logger.exception(err, True, f"Error when cancelling order [{order}]: {err}")
+                        failed_to_cancel = True
+        return cancelled and not failed_to_cancel
 
     def all_databases(self):
         provider = databases.RunDatabasesProvider.instance()
@@ -445,20 +476,25 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
             self.logger.error(f"Initialization took more than {timeout} seconds")
         return False
 
-    async def _wait_for_bot_init(self, timeout, extra_topics: list=None) -> bool:
+    async def _wait_for_bot_init(self, timeout, extra_symbol_topics: list=None, extra_global_topics: list=None) -> bool:
         try:
-            topics = [
-                common_enums.InitializationEventExchangeTopics.BALANCE.value,
-                common_enums.InitializationEventExchangeTopics.ORDERS.value
-            ] + (extra_topics if extra_topics else [])
+            init_symbols = self.trading_mode.get_init_symbols()
+            symbol_topics = [common_enums.InitializationEventExchangeTopics.ORDERS.value] + (extra_symbol_topics or [])
             if self.trading_mode.REQUIRE_TRADES_HISTORY:
-                topics.append(common_enums.InitializationEventExchangeTopics.TRADES.value)
-            for topic in topics:
+                symbol_topics.append(common_enums.InitializationEventExchangeTopics.TRADES.value)
+            for symbol_topic in symbol_topics:
                 self.logger.debug(f"Trading mode [{self.exchange_manager.exchange_name}] start complete. "
-                                  f"Now waiting for {topic} full initialisation.")
-                await util.wait_for_topic_init(self.exchange_manager, timeout, topic)
+                                  f"1/2: Now waiting for {symbol_topic} (for {init_symbols}) full initialisation.")
+                await util.wait_for_topic_init(
+                    self.exchange_manager, timeout, symbol_topic, symbols=init_symbols
+                )
+            global_topics = [common_enums.InitializationEventExchangeTopics.BALANCE.value] + (extra_global_topics or [])
+            for global_topic in global_topics:
+                self.logger.debug(f"Trading mode [{self.exchange_manager.exchange_name}] start complete. "
+                                  f"2/2: Now waiting for {global_topics} full initialisation.")
+                await util.wait_for_topic_init(self.exchange_manager, timeout, global_topic)
             self.logger.debug(
-                f"Trading mode requirements init complete: {', '.join(t for t in topics)} initialisation completed."
+                f"Trading mode requirements init complete: {', '.join(t for t in symbol_topics)} initialisation completed."
             )
             return True
         except (asyncio.TimeoutError, concurrent.futures.TimeoutError):
@@ -487,20 +523,41 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
                 common_enums.ActivationTopics.EVALUATION_CYCLE.value,
                 activation_topic_values
             )
-        try:
-            await self._apply_exchange_side_config(context)
-        except Exception as err:
-            # TODO important error to display
-            self.logger.exception(err, True, f"Error when applying exchange side config: {err}")
+        if self.trading_mode.is_updating_exchange_settings(context):
+            try:
+                await self._apply_exchange_side_config(context)
+            except Exception as err:
+                # TODO important error to display
+                self.logger.exception(err, True, f"Error when applying exchange side config: {err}")
 
     async def _apply_exchange_side_config(self, context):
         # can be slow, call in a task if necessary
         if context.exchange_manager.is_future:
             if not self._is_ready_to_trade.is_set():
-                await util.wait_for_topic_init(self.exchange_manager, self.CONFIG_INIT_TIMEOUT,
-                                               common_enums.InitializationEventExchangeTopics.CONTRACTS.value)
+                await util.wait_for_topic_init(
+                    self.exchange_manager, self._get_config_init_timeout(),
+                    common_enums.InitializationEventExchangeTopics.CONTRACTS.value,
+                    symbols=self.trading_mode.get_init_symbols()
+                )
             await script_keywords.set_leverage(context, await script_keywords.user_select_leverage(context))
 
+    async def _wait_for_symbol_prices_and_profitability_init(self, timeout) -> bool:
+        try:
+            await util.wait_for_topic_init(
+                self.exchange_manager, timeout, common_enums.InitializationEventExchangeTopics.PRICE.value,
+                symbols=self.trading_mode.get_init_symbols()
+            )
+            await util.wait_for_topic_init(self.exchange_manager, timeout,
+                                           common_enums.InitializationEventExchangeTopics.PROFITABILITY.value)
+        except (asyncio.TimeoutError, concurrent.futures.TimeoutError):
+            self.logger.error(f"Symbol price initialization took more than {timeout} seconds")
+        return False
+
+    def _get_config_init_timeout(self) -> int:
+        if self.exchange_manager and self.exchange_manager.is_backtesting:
+            return 1    # in backesting, init should be almost instant. If it's not, wait for yp to 1 second
+        return 1 * common_constants.MINUTE_TO_SECONDS  # let time for orders to be fetched before
+
     @classmethod
     def producer_exchange_wide_lock(cls, exchange_manager) -> asyncio_tools.RLock():
         try:
@@ -516,3 +573,19 @@ class AbstractTradingModeProducer(modes_channel.ModeChannelProducer):
             cls.PRODUCER_LOCKS_BY_EXCHANGE_ID.pop(
                 exchange_manager.id, None
             )
+
+    async def sent_once_critical_notification(self, title: str, description: str):
+        if description not in self._sent_critical_notifications:
+            try:
+                import octobot_services.api as services_api
+                import octobot_services.enums as services_enum
+                self.logger.error(f"Critical error: {title}: {description}")
+                await services_api.send_notification(services_api.create_notification(
+                    description, title=title, markdown_text=description,
+                    level=services_enum.NotificationLevel.CRITICAL,
+                    category=services_enum.NotificationCategory.GLOBAL_INFO
+                ))
+            except ImportError as e:
+                self.logger.exception(e, True, f"Impossible to send notification: {e}")
+            finally:
+                self._sent_critical_notifications.add(description)
