@@ -14,7 +14,9 @@
 #  You should have received a copy of the GNU Lesser General Public
 #  License along with this library.
 import asyncio
+import decimal
 import uuid
+import typing
 
 import octobot_commons.logging as logging
 
@@ -28,6 +30,7 @@ import octobot_trading.personal_data.orders.orders_manager as orders_manager
 import octobot_trading.personal_data.orders.order as order_import
 import octobot_trading.personal_data.orders.orders_storage_operations as orders_storage_operations
 import octobot_trading.personal_data.trades.trades_manager as trades_manager
+import octobot_trading.personal_data.trades.trade as trade_import
 import octobot_trading.personal_data.transactions.transactions_manager as transactions_manager
 import octobot_trading.personal_data.transactions.transaction_factory as transaction_factory
 import octobot_trading.util as util
@@ -44,19 +47,20 @@ class ExchangePersonalData(util.Initializable):
         self.trader = None
         self.exchange = None
 
-        self.portfolio_manager = None
-        self.trades_manager = None
-        self.orders_manager = None
-        self.positions_manager = None
-        self.transactions_manager = None
+        self.portfolio_manager: portfolio_manager.PortfolioManager = None
+        self.trades_manager: trades_manager.TradesManager = None
+        self.orders_manager: orders_manager.OrdersManager = None
+        self.positions_manager: positions_manager.PositionsManager = None
+        self.transactions_manager: transactions_manager.TransactionsManager = None
 
     async def initialize_impl(self):
         self.trader = self.exchange_manager.trader
         self.exchange = self.exchange_manager.exchange
         if self.trader.is_enabled:
             try:
-                self.portfolio_manager = portfolio_manager.PortfolioManager(self.config, self.trader,
-                                                                            self.exchange_manager)
+                self.portfolio_manager = portfolio_manager.PortfolioManager(
+                    self.config, self.trader, self.exchange_manager
+                )
                 self.trades_manager = trades_manager.TradesManager(self.trader)
                 self.orders_manager = orders_manager.OrdersManager(self.trader)
                 self.positions_manager = positions_manager.PositionsManager(self.trader)
@@ -83,11 +87,12 @@ class ExchangePersonalData(util.Initializable):
             return False
 
     async def handle_portfolio_and_position_update_from_order(
-        self, order, require_exchange_update: bool = True, should_notify: bool = True
+        self, order, require_exchange_update: bool = True, expect_filled_order_update: bool = False,
+        should_notify: bool = True
     ) -> bool:
         try:
             changed: bool = await self.portfolio_manager.handle_balance_update_from_order(
-                order, require_exchange_update
+                order, require_exchange_update, expect_filled_order_update
             )
             if self.exchange_manager.is_future:
                 changed = await self.positions_manager.handle_position_update_from_order(
@@ -97,7 +102,6 @@ class ExchangePersonalData(util.Initializable):
                 await self.handle_portfolio_update_notification(self.portfolio_manager.portfolio.portfolio)
 
                 if self.exchange_manager.is_future:
-                    # should this be done only "if should_notify" ?
                     await self.handle_position_instance_update(
                         order.exchange_manager.exchange_personal_data.positions_manager.get_order_position(order),
                         should_notify=True
@@ -185,6 +189,8 @@ class ExchangePersonalData(util.Initializable):
         if self._is_out_of_sync_order(exchange_order_id):
             self.logger.debug(f"Ignored update for order with exchange id: {exchange_order_id}: this order "
                               f"has already been closed (received raw order: {raw_order})")
+            # ensure order is not in open orders anymore
+            self._ensure_canceled_out_of_sync_order(exchange_order_id)
         else:
             try:
                 changed, order = await self.orders_manager.upsert_order_from_raw(
@@ -206,6 +212,19 @@ class ExchangePersonalData(util.Initializable):
                 self.logger.exception(e, True, f"Failed to update order : {e}")
         return False, None
 
+    def _ensure_canceled_out_of_sync_order(self, exchange_order_id: str):
+        try:
+            out_of_sync_open_order = self.orders_manager.get_order(None, exchange_order_id=exchange_order_id)
+            # order should be removed from open orders to avoid looping in refresh
+            self.orders_manager.remove_order_instance(out_of_sync_open_order)
+            self.logger.info(
+                f"Out of sync order with exchange id {exchange_order_id} has been cancelled. "
+                f"Order: {out_of_sync_open_order}."
+            )
+        except KeyError:
+            # order is not in open orders anymore: nothing to do
+            pass
+
     async def update_order_from_stored_data(self, exchange_order_id, pending_groups):
         order = self.orders_manager.get_order(None, exchange_order_id=exchange_order_id)
         previous_order_id = order.order_id
@@ -276,6 +295,24 @@ class ExchangePersonalData(util.Initializable):
             self.logger.exception(e, True, f"Failed to update order : {e}")
             return False
 
+    async def check_and_update_inactive_orders_when_necessary(
+        self, symbol: str, current_price: decimal.Decimal, price_time: float,
+        strategy_timeout: typing.Optional[float], wait_for_fill_callback: typing.Optional[typing.Callable]
+    ) -> int:
+        handled_orders_count = 0
+        sorted_inactive_orders = sorted(
+            self.orders_manager.get_all_orders(symbol=symbol, active=False),
+            key= lambda o: o.origin_price if o.trigger_above else -o.origin_price
+        )
+        for order in sorted_inactive_orders:
+            if order.should_become_active(price_time, current_price):
+                handled_orders_count += 1
+                try:
+                    await order.on_active_trigger(strategy_timeout, wait_for_fill_callback)
+                except Exception as err:
+                    self.logger.exception(err, True, f"Failed order on_active_trigger {err} (order: {order})")
+        return handled_orders_count
+
     async def handle_trade_update(self, symbol, trade_id, trade,
                                   is_old_trade: bool = False, should_notify: bool = True):
         try:
@@ -360,6 +397,16 @@ class ExchangePersonalData(util.Initializable):
         except ValueError as e:
             self.logger.error(f"Failed to send position update notification : {e}")
 
+    def get_trade_or_open_order(self, order_id: str) -> (
+        typing.Optional[trade_import.Trade], typing.Optional[order_import.Order]
+    ):
+        trade = self.trades_manager.get_trade_from_order_id(order_id)
+        try:
+            order = self.orders_manager.get_order(order_id)
+        except KeyError:
+            order = None
+        return trade, order
+
     async def stop(self):
         if self.portfolio_manager is not None:
             await self.portfolio_manager.stop()
