@@ -1,3 +1,4 @@
+# pylint: disable=W0706
 #  Drakkar-Software OctoBot-Trading
 #  Copyright (c) Drakkar-Software, All rights reserved.
 #
@@ -17,6 +18,7 @@ import asyncio
 import decimal
 import contextlib
 import uuid
+import typing
 
 import octobot_commons.symbols as symbol_util
 import octobot_commons.constants as commons_constants
@@ -28,20 +30,24 @@ import octobot_trading.errors as errors
 import octobot_trading.personal_data.orders.decimal_order_adapter as decimal_order_adapter
 import octobot_trading.exchanges.util.exchange_market_status_fixer as exchange_market_status_fixer
 import octobot_trading.personal_data.orders.states.fill_order_state as fill_order_state
+import octobot_trading.personal_data.orders.order as order_import
+import octobot_trading.personal_data.orders.triggers.price_trigger as price_trigger
+import octobot_trading.signals as signals
 from octobot_trading.enums import ExchangeConstantsMarketStatusColumns as Ecmsc
 
 
 LOGGER_NAME = "order_util"
 
 
-def is_valid(element, key):
+def is_valid(element, key, zero_valid=False):
     """
     Checks is the element is valid with the market status fixer
     :param element:
     :param key:
+    :param zero_valid: if 0 should be considered a valid value
     :return:
     """
-    return key in element and exchange_market_status_fixer.is_ms_valid(element[key])
+    return key in element and exchange_market_status_fixer.is_ms_valid(element[key], zero_valid=zero_valid)
 
 
 def get_min_max_amounts(symbol_market, default_value=None):
@@ -105,10 +111,53 @@ def check_cost(total_order_price, min_cost):
     return True
 
 
+def get_valid_split_orders(
+    quantity: decimal.Decimal, prices: list[decimal.Decimal], symbol_market,
+    amount_ratio_per_order: typing.Optional[list[decimal.Decimal]] = None
+) -> (list[decimal.Decimal], list[decimal.Decimal]):
+    if amount_ratio_per_order:
+        if len(amount_ratio_per_order) != len(prices):
+            raise ValueError(f"amount_ratio_per_order must have {len(prices)} elements")
+        if any(amount_ratio <= constants.ZERO for amount_ratio in amount_ratio_per_order):
+            raise ValueError(f"all amount_ratio_per_order must by > {constants.ZERO}")
+    if len(prices) < 1:
+        return [], []
+    if len(prices) < 2:
+        return [quantity], [prices[0]]
+    min_quantity, max_quantity, min_cost, max_cost, min_price, max_price = get_min_max_amounts(symbol_market)
+    min_quantity = None if min_quantity is None else decimal.Decimal(f"{min_quantity}")
+    max_quantity = None if max_quantity is None else decimal.Decimal(f"{max_quantity}")
+    min_cost = None if min_cost is None else decimal.Decimal(f"{min_cost}")
+    max_cost = None if max_cost is None else decimal.Decimal(f"{max_cost}")
+    min_price = None if min_price is None else decimal.Decimal(f"{min_price}")
+    max_price = None if max_price is None else decimal.Decimal(f"{max_price}")
+    # try to split quantity evenly amount prices
+    current_supported_orders_count = decimal.Decimal(str(len(prices)))
+    while current_supported_orders_count > constants.ONE:
+        if amount_ratio_per_order:
+            used_amount_ratio_per_order = amount_ratio_per_order[:int(current_supported_orders_count)]
+            total_ratios = sum(used_amount_ratio_per_order)
+            candidate_volumes = [
+                quantity * ratio_per_order / total_ratios
+                for ratio_per_order in used_amount_ratio_per_order
+            ]
+        else:
+            candidate_volumes = [quantity / current_supported_orders_count] * int(current_supported_orders_count)
+        candidate_prices = prices[:int(current_supported_orders_count)]
+        if _are_orders_too_small(min_quantity, min_cost, min_price, min(candidate_prices), min(candidate_volumes)):
+            # reduce orders count to increase quantity
+            current_supported_orders_count -= 1
+        elif _are_orders_too_large(max_quantity, max_cost, max_price, max(candidate_prices), max(candidate_volumes)):
+            raise errors.InvalidArgumentError(f"Order volume ({max(candidate_volumes)}) is too large")
+        else:
+            return candidate_volumes, candidate_prices
+    # default to full quantity and first price
+    return [quantity], [prices[0]]
+
+
 def get_split_orders_count_and_increment(
     lower_price, higher_price, quantity, orders_count, symbol_market, add_increment_to_min_price
-) \
-        -> (int, decimal.Decimal):
+) -> (int, decimal.Decimal):
     """
     :param lower_price: smallest order price
     :param higher_price: largest order price
@@ -201,6 +250,17 @@ async def get_up_to_date_price(exchange_manager, symbol: str, timeout: int = Non
     return decimal.Decimal(str(mark_price))
 
 
+def get_potentially_outdated_price(exchange_manager, symbol: str) -> (decimal.Decimal, bool):
+    symbol_data = exchange_manager.exchange_symbols_data.get_exchange_symbol_data(
+        symbol, allow_creation=False
+    )
+    try:
+        return symbol_data.prices_manager.get_mark_price_no_wait(), True
+    except ValueError:
+        # outdated price, return it anyway
+        return symbol_data.prices_manager.mark_price, False
+
+
 async def get_pre_order_data(exchange_manager, symbol: str, timeout: int = None,
                              portfolio_type=commons_constants.PORTFOLIO_AVAILABLE,
                              target_price=None):
@@ -242,6 +302,9 @@ def get_futures_max_order_size(exchange_manager, symbol, side, current_price, re
         symbol,
         enums.PositionSide.BOTH
     )
+    if reduce_only and current_position.is_idle():
+        # can't reduce an empty position
+        return constants.ZERO, False
     # ensure max position order size is taken into account
     new_position_side = current_position.side
     if new_position_side is enums.PositionSide.UNKNOWN:
@@ -250,24 +313,35 @@ def get_futures_max_order_size(exchange_manager, symbol, side, current_price, re
 
     contract_current_symbol_holding = current_symbol_holding
     contract_market_quantity = market_quantity
+    added_quantity_from_reverse = constants.ZERO
+    might_reverse_position = False
     if current_position.symbol_contract.is_inverse_contract():
         # use USD (in BTC/USD) for order sizes on inverse, convert BTC values into USD ones
         contract_current_symbol_holding = current_symbol_holding * current_price
         contract_market_quantity = market_quantity * current_price
     if side is enums.TradeOrderSide.SELL and current_position.is_long():
         # can also sell the position size in long
-        contract_current_symbol_holding = current_position.size if reduce_only \
-            else contract_current_symbol_holding + current_position.size
+        contract_current_symbol_holding = current_position.size
+        might_reverse_position = not reduce_only
+        added_quantity_from_reverse = current_position.initial_margin
     elif side is enums.TradeOrderSide.BUY and current_position.is_short():
         # can also buy the position size in short
-        contract_market_quantity = abs(current_position.size) if reduce_only \
-            else contract_market_quantity + abs(current_position.size)
-    if (new_position_side is enums.PositionSide.LONG and side is enums.TradeOrderSide.BUY) \
-            or (new_position_side is enums.PositionSide.SHORT and
-                side is enums.TradeOrderSide.SELL):
+        contract_market_quantity = abs(current_position.size)
+        might_reverse_position = not reduce_only
+        added_quantity_from_reverse = current_position.initial_margin
+    max_reducing_position_order_size = (
+        contract_market_quantity if side is enums.TradeOrderSide.BUY else contract_current_symbol_holding
+    )
+    if (
+        (new_position_side is enums.PositionSide.LONG and side is enums.TradeOrderSide.BUY)
+        or (new_position_side is enums.PositionSide.SHORT and  side is enums.TradeOrderSide.SELL)
+        or might_reverse_position
+    ):
         quantity = market_quantity if current_position.symbol_contract.is_inverse_contract() \
             else market_quantity * current_price
-        unleveraged_quantity = quantity / current_position.symbol_contract.current_leverage
+        # include added_quantity_from_reverse in case funds get freed from reversing a position
+        usable_quantity = quantity + added_quantity_from_reverse
+        unleveraged_quantity = usable_quantity / current_position.symbol_contract.current_leverage
         max_position_increased_order_quantity = get_max_order_quantity_for_price(
             current_position, unleveraged_quantity, current_price, new_position_side, symbol
         )
@@ -278,8 +352,11 @@ def get_futures_max_order_size(exchange_manager, symbol, side, current_price, re
             max_position_increased_order_quantity *= \
                 exchange_manager.exchange.MAX_INCREASED_POSITION_QUANTITY_MULTIPLIER
         # increasing position: always use the same currency
-        return max_position_increased_order_quantity, True
-    return contract_market_quantity if side is enums.TradeOrderSide.BUY else contract_current_symbol_holding, False
+        return max_position_increased_order_quantity + (
+            # include reducing position amount to process reverse
+            max_reducing_position_order_size if might_reverse_position else constants.ZERO
+        ), True
+    return max_reducing_position_order_size, False
 
 
 def get_max_order_quantity_for_price(position, available_quantity, price, side, symbol):
@@ -307,6 +384,18 @@ def get_max_order_quantity_for_price(position, available_quantity, price, side,
         (two_way_fees + price)
 
 
+def get_locked_funds(order):
+    forecasted_fees = order.get_computed_fee(use_origin_quantity_and_price=not order.is_filled())
+    if order.side == enums.TradeOrderSide.BUY:
+        # locking quote to buy
+        quote_fees = get_fees_for_currency(forecasted_fees, order.market)
+        return order.origin_quantity * order.origin_price + quote_fees
+    else:
+        # locking base to sell
+        base_fees = get_fees_for_currency(forecasted_fees, order.currency)
+        return order.origin_quantity + base_fees
+
+
 def total_fees_from_order_dict(order_dict, currency):
     return get_fees_for_currency(order_dict[enums.ExchangeConstantsOrderColumns.FEE.value], currency)
 
@@ -317,6 +406,43 @@ def get_fees_for_currency(fee, currency):
     return constants.ZERO
 
 
+def get_order_locked_amount(order: order_import.Order, force_use_origin_quantity_and_price=False) -> decimal.Decimal:
+    # take fees into account when in locked asset
+    # ( a BTC/USDT order with USDT fees need to lock USDT fees to be able to pay them)
+    use_origin_quantity_and_price = force_use_origin_quantity_and_price or not order.is_filled()
+    forecasted_fees = order.get_computed_fee(use_origin_quantity_and_price=use_origin_quantity_and_price)
+    base, quote = symbol_util.parse_symbol(order.symbol).base_and_quote()
+    # when buy order
+    if order.side == enums.TradeOrderSide.BUY:
+        return order.origin_quantity * order.origin_price + get_fees_for_currency(forecasted_fees, quote)
+    # when sell order
+    return order.origin_quantity + get_fees_for_currency(forecasted_fees, base)
+
+
+def get_orders_locked_amounts_by_asset(open_orders: list[order_import.Order]) -> dict[str, decimal.Decimal]:
+    if not open_orders:
+        return {}
+    locked_funds_by_asset = {}
+    for order in open_orders:
+        base, quote = symbol_util.parse_symbol(order.symbol).base_and_quote()
+        # use get_order_locked_amount just like trader simulator to ensure locked funds integrity
+        if order.side == enums.TradeOrderSide.BUY:
+            # buy orders only lock fees in quote
+            locked_quote = get_order_locked_amount(order)
+            if quote not in locked_funds_by_asset:
+                locked_funds_by_asset[quote] = locked_quote
+            else:
+                locked_funds_by_asset[quote] += locked_quote
+        else:
+            # sell orders only lock fees in base
+            locked_base = get_order_locked_amount(order)
+            if base not in locked_funds_by_asset:
+                locked_funds_by_asset[base] = locked_base
+            else:
+                locked_funds_by_asset[base] += locked_base
+    return locked_funds_by_asset
+
+
 def parse_raw_fees(raw_fees):
     fees = raw_fees
     if fees:
@@ -351,6 +477,10 @@ def parse_is_cancelled(raw_order):
     return parse_order_status(raw_order) in {enums.OrderStatus.CANCELED, enums.OrderStatus.CLOSED}
 
 
+def parse_is_pending_cancel(raw_order):
+    return parse_order_status(raw_order) is enums.OrderStatus.PENDING_CANCEL
+
+
 def parse_is_open(raw_order):
     return parse_order_status(raw_order) is enums.OrderStatus.OPEN
 
@@ -374,12 +504,41 @@ def is_stop_order(order_type: enums.TraderOrderType):
     ]
 
 
-def is_take_profit_order(order_type):
+def is_stop_trade_order_type(order_type: enums.TradeOrderType):
+    return order_type in [
+        enums.TradeOrderType.STOP_LOSS, enums.TradeOrderType.STOP_LOSS_LIMIT,
+        enums.TradeOrderType.TRAILING_STOP, enums.TradeOrderType.TRAILING_STOP_LIMIT,
+    ]
+
+
+def is_take_profit_order(order_type: enums.TraderOrderType):
     return order_type in [
         enums.TraderOrderType.TAKE_PROFIT, enums.TraderOrderType.TAKE_PROFIT_LIMIT,
     ]
 
 
+def ensure_orders_limit(exchange_manager, symbol, added_orders: list[enums.TraderOrderType]):
+    max_limit_orders = exchange_manager.exchange.get_max_orders_count(symbol, enums.TraderOrderType.SELL_LIMIT)
+    max_stop_orders = exchange_manager.exchange.get_max_orders_count(symbol, enums.TraderOrderType.STOP_LOSS)
+    open_orders = exchange_manager.exchange_personal_data.orders_manager.get_open_orders(
+        symbol=symbol, active=None  # consider both active and inactive orders of this symbol
+    )
+    stop_orders_count = len([o for o in open_orders if is_stop_order(o.order_type)]) + len(
+        [o_type for o_type in added_orders if is_stop_order(o_type)]
+    )
+    limit_orders_count = len(open_orders) + len(added_orders) - stop_orders_count
+    if limit_orders_count > max_limit_orders:
+        raise errors.MaxOpenOrderReachedForSymbolError(
+            f"{symbol} max open limit orders reached: "
+            f"limit = {max_limit_orders}, orders count = {limit_orders_count}"
+        )
+    if stop_orders_count > max_stop_orders:
+        raise errors.MaxOpenOrderReachedForSymbolError(
+            f"{symbol} max open stop orders reached: "
+            f"limit = {max_stop_orders}, orders count = {stop_orders_count}"
+        )
+
+
 def get_trade_order_type(order_type: enums.TraderOrderType) -> enums.TradeOrderType:
     if order_type in (enums.TraderOrderType.BUY_MARKET, enums.TraderOrderType.SELL_MARKET):
         return enums.TradeOrderType.MARKET
@@ -400,6 +559,58 @@ def get_trade_order_type(order_type: enums.TraderOrderType) -> enums.TradeOrderT
     raise ValueError(order_type)
 
 
+def create_order_price_trigger(
+    order: order_import.Order, active_trigger_price: decimal.Decimal, active_trigger_above: bool
+) -> price_trigger.PriceTrigger:
+    if active_trigger_price is None or active_trigger_above is None:
+        raise ValueError("active_trigger_price and active_trigger_above must be specified")
+    return price_trigger.PriceTrigger(
+        order.on_active_trigger, (None, None), active_trigger_price, active_trigger_above
+    )
+
+
+async def create_as_active_order_using_strategy_if_any(
+    order, strategy_timeout: typing.Optional[float], wait_for_fill_callback: typing.Optional[typing.Callable]
+):
+    if active_swap_strategy := order.order_group.active_order_swap_strategy if order.order_group else None:
+        # use strategy to process swap
+        await active_swap_strategy.execute(order, wait_for_fill_callback, strategy_timeout)
+    else:
+        # no strategy: just create this order as active
+        await create_as_active_order_on_exchange(order, False)
+
+
+async def create_as_active_order_on_exchange(order_to_create, emit_trading_signals: bool):
+    async with signals.remote_signal_publisher(
+        order_to_create.trader.exchange_manager, order_to_create.symbol, emit_trading_signals
+    ):
+        active_order = await signals.update_order_as_active(
+            order_to_create.trader.exchange_manager,
+            signals.should_emit_trading_signal(order_to_create.trader.exchange_manager),
+            order_to_create,
+            params=order_to_create.exchange_creation_params
+        )
+        if active_order is None:
+            logging.get_logger(order_to_create.get_logger_name()).error(
+                f"Failed to create active order for {order_to_create}"
+            )
+        return active_order
+
+
+async def update_order_as_inactive_on_exchange(order_to_cancel, emit_trading_signals: bool):
+    async with signals.remote_signal_publisher(
+        order_to_cancel.trader.exchange_manager, order_to_cancel.symbol, emit_trading_signals
+    ):
+        if not await signals.update_order_as_inactive(
+            order_to_cancel.trader.exchange_manager,
+            signals.should_emit_trading_signal(order_to_cancel.trader.exchange_manager),
+            order_to_cancel,
+        ):
+            logging.get_logger(order_to_cancel.get_logger_name()).error(f"Failed to cancel active order on exchange: {order_to_cancel}")
+            return False
+    return True
+
+
 async def create_as_chained_order(order):
     order.is_waiting_for_chained_trigger = False
     if not order.trader.simulate and order.has_been_bundled:
@@ -421,8 +632,12 @@ async def create_as_chained_order(order):
                 order,
                 loaded=False,
                 params=order.exchange_creation_params,
+                raise_all_creation_error=True,
                 **order.trader_creation_kwargs
             )
+        except (errors.ExchangeClosedPositionError, errors.ExchangeOrderInstantTriggerError):
+            # Order can be created and might be outdated forward error for the caller to fix it if possible
+            raise
         except Exception as err:
             # log warning to be sure to keep track of the failed order details
             logging.get_logger(LOGGER_NAME).warning(
@@ -539,6 +754,46 @@ def generate_order_id():
     return str(uuid.uuid4())
 
 
+def _get_possible_filled_price(
+    exchange_manager, symbol: str, side: enums.TradeOrderSide, ideal_price: decimal.Decimal
+) -> decimal.Decimal:
+    try:
+        min_price, max_price = exchange_manager.exchange_symbols_data.get_exchange_symbol_data(symbol).\
+            price_events_manager.get_min_and_max_prices()
+        if side is enums.TradeOrderSide.SELL:
+            if ideal_price < min_price:
+                return min_price
+        if side is enums.TradeOrderSide.BUY:
+            if ideal_price > max_price:
+                return max_price
+    except IndexError:
+        # no available price data
+        pass
+    return ideal_price
+
+
+def get_valid_filled_price(order, ideal_price: decimal.Decimal):
+    # ensure filled price based on ideal_price makes sense according to the current candle
+    # instead of blindly using ideal_price as this could
+    # potentially buy higher than the candle high or sell lower than the candle low,
+    # which can't happen on real conditions
+
+    if order.exchange_manager.is_backtesting:
+        # in backtesting: ensure ideal_price is with in current candle price.
+        return _get_possible_filled_price(order.exchange_manager, order.symbol, order.side, ideal_price)
+    # not in backtesting: price desync can't happen
+    return ideal_price
+
+
+async def adapt_chained_order_before_creation(base_order, chained_order):
+    can_be_created = True
+    if chained_order.update_with_triggering_order_fees:
+        can_be_created = chained_order.update_quantity_with_order_fees(base_order)
+    # ensure price is not outdated
+    await chained_order.update_price_if_outdated()
+    return can_be_created
+
+
 async def wait_for_order_fill(order, timeout, wait_for_portfolio_update):
     if order.is_open():
         if order.state is None:
