@@ -14,12 +14,14 @@
 #  You should have received a copy of the GNU Lesser General Public
 #  License along with this library.
 import copy
+import typing
 
 import octobot_commons.logging as logging
 import octobot_commons.symbols as symbol_util
 
 import octobot_trading.constants as constants
 import octobot_trading.errors as errors
+import octobot_trading.enums as enums
 import octobot_trading.personal_data.portfolios.value_converter as value_converter
 
 
@@ -116,17 +118,61 @@ class PortfolioValueHolder:
             for currency in holdings.keys()
         }
 
-    def get_currency_holding_ratio(self, currency):
-        """
-        Return the holdings ratio for the specified currency
-        :return: the holdings ratio
-        """
-        if self.portfolio_current_value:
-            return self.value_converter.evaluate_value(
-                currency,
-                self.portfolio_manager.portfolio.get_currency_portfolio(currency).total
-            ) / self.portfolio_current_value
-        return constants.ZERO
+    def get_assets_holdings_value(self, assets, target_unit, init_price_fetchers=False):
+        total_value = constants.ZERO
+        for asset, asset_holdings in self.portfolio_manager.portfolio.portfolio.items():
+            if asset not in assets:
+                continue
+            try:
+                total_value += self.value_converter.evaluate_value(
+                    asset, asset_holdings.total, raise_error=True,
+                    target_currency=target_unit, init_price_fetchers=init_price_fetchers
+                )
+            except errors.MissingPriceDataError:
+                self.logger.info(f"Missing {asset} price conversion, ignoring {float(asset_holdings.total)} holdings.")
+        return total_value
+
+    def get_traded_assets_holdings_value(self, unit: str, coins_whitelist: typing.Optional[typing.Iterable]):
+        assets = set()
+        for symbol in self.portfolio_manager.exchange_manager.exchange_config.traded_symbols:
+            if coins_whitelist is None or symbol.base in coins_whitelist:
+                assets.add(symbol.base)
+            if coins_whitelist is None or symbol.quote in coins_whitelist:
+                assets.add(symbol.quote)
+        return self.get_assets_holdings_value(
+            assets, unit
+        )
+
+    def get_holdings_ratio(
+        self, currency, traded_symbols_only=False, include_assets_in_open_orders=False, coins_whitelist=None
+    ):
+        if coins_whitelist and not traded_symbols_only:
+            total_holdings_value = self.get_assets_holdings_value(
+                coins_whitelist, self.portfolio_manager.reference_market
+            )
+        elif traded_symbols_only:
+            total_holdings_value = self.get_traded_assets_holdings_value(
+                self.portfolio_manager.reference_market, coins_whitelist
+            )
+        else:
+            # consider all assets for total_holdings_value
+            total_holdings_value = self.portfolio_current_value
+        if not total_holdings_value:
+            return constants.ZERO
+        currency_holdings = self.portfolio_manager.portfolio.get_currency_portfolio(currency).total
+        if include_assets_in_open_orders:
+            # add assets in open orders to currency_holdings
+            assets_in_open_orders = constants.ZERO
+            for order in self.portfolio_manager.exchange_manager.exchange_personal_data.orders_manager.get_open_orders():
+                symbol = symbol_util.parse_symbol(order.symbol)
+                if order.side is enums.TradeOrderSide.BUY and symbol.base == currency:
+                    assets_in_open_orders += order.origin_quantity
+                elif order.side is enums.TradeOrderSide.SELL and symbol.quote == currency:
+                    assets_in_open_orders += order.total_cost
+            currency_holdings += assets_in_open_orders
+        # compute ratio
+        current_holdings_value = self.value_converter.evaluate_value(currency, currency_holdings)
+        return current_holdings_value / total_holdings_value
 
     def handle_profitability_recalculation(self, force_recompute_origin_portfolio):
         """
