@@ -18,16 +18,21 @@ import os
 import ccxt.async_support
 import decimal
 
+import mock
 import pytest
 import time
 from mock import AsyncMock, patch, Mock
 
-from octobot_trading.errors import TooManyOpenPositionError, InvalidLeverageValue, OrderEditError
+from octobot_trading.errors import (
+    TooManyOpenPositionError, InvalidLeverageValue, OrderEditError,
+    ExchangeOrderCancelError, OrderNotFoundOnCancelError
+)
 from octobot_trading.personal_data import LinearPosition
 import octobot_commons.constants as commons_constants
 from octobot_commons.asyncio_tools import wait_asyncio_next_cycle
 from octobot_commons.tests.test_config import load_test_config
 from octobot_trading.personal_data.orders import Order
+import octobot_trading.errors as errors
 from octobot_trading.enums import TraderOrderType, TradeOrderSide, TradeOrderType, OrderStatus, \
     ExchangeConstantsPositionColumns, PositionMode, MarginType, TakeProfitStopLossMode, ExchangeSupportedElements
 from octobot_trading.exchanges.exchange_manager import ExchangeManager
@@ -62,7 +67,7 @@ class TestTrader:
                                            DEFAULT_FUTURE_EXCHANGE_NAME if is_future else DEFAULT_EXCHANGE_NAME)
         exchange_manager.is_simulated = simulated
         exchange_manager.is_future = is_future
-        await exchange_manager.initialize()
+        await exchange_manager.initialize(exchange_config_by_exchange=None)
 
         trader = TraderSimulator(config, exchange_manager)
         await trader.initialize()
@@ -176,6 +181,70 @@ class TestTrader:
 
         await self.stop(exchange_manager)
 
+
+    async def test_cancel_order_error(self):
+        _, exchange_manager, trader_inst = await self.init_default()
+
+        limit_buy = BuyLimitOrder(trader_inst)
+        limit_buy.update(order_type=TraderOrderType.SELL_LIMIT,
+                         symbol=self.DEFAULT_SYMBOL,
+                         current_price=decimal.Decimal("70"),
+                         quantity=decimal.Decimal("10"),
+                         price=decimal.Decimal("70"))
+        try:
+            trader_inst.simulate = False
+            with mock.patch.object(trader_inst, "_handle_order_cancel_error", mock.AsyncMock(
+                return_value=True
+            )) as _handle_order_cancel_error_mock:
+                with mock.patch.object(exchange_manager.exchange, "cancel_order", mock.AsyncMock(
+                    return_value=OrderStatus.CANCELED
+                )) as cancel_order_mock:
+                    assert await trader_inst.cancel_order(limit_buy) is True
+                    cancel_order_mock.assert_called_once()
+                    _handle_order_cancel_error_mock.assert_not_called()
+                    limit_buy.state = None
+                with mock.patch.object(exchange_manager.exchange, "cancel_order", mock.AsyncMock(
+                    side_effect=ExchangeOrderCancelError
+                )) as cancel_order_mock:
+                    assert await trader_inst.cancel_order(limit_buy) is True
+                    assert cancel_order_mock.call_count == 2
+                    _handle_order_cancel_error_mock.assert_called_once()
+                    _handle_order_cancel_error_mock.reset_mock()
+                    limit_buy.state = None
+                with mock.patch.object(exchange_manager.exchange, "cancel_order", mock.AsyncMock(
+                    side_effect=OrderNotFoundOnCancelError
+                )) as cancel_order_mock:
+                    assert await trader_inst.cancel_order(limit_buy) is True
+                    assert cancel_order_mock.call_count == 2
+                    _handle_order_cancel_error_mock.assert_called_once()
+                    _handle_order_cancel_error_mock.reset_mock()
+                    limit_buy.state = None
+
+                with trader_inst.exchange_manager.exchange_personal_data.orders_manager.disabled_order_auto_synchronization():
+                    with mock.patch.object(exchange_manager.exchange, "cancel_order", mock.AsyncMock(
+                        side_effect=OrderNotFoundOnCancelError
+                    )) as cancel_order_mock:
+                        assert await trader_inst.cancel_order(limit_buy) is True
+                        assert cancel_order_mock.call_count == 2
+                        _handle_order_cancel_error_mock.assert_not_called()
+                        limit_buy.state = None
+
+            with mock.patch.object(trader_inst, "_handle_order_cancel_error", mock.AsyncMock(
+                return_value=False
+            )) as _handle_order_cancel_error_mock:
+                with mock.patch.object(exchange_manager.exchange, "cancel_order", mock.AsyncMock(
+                    side_effect=ExchangeOrderCancelError
+                )) as cancel_order_mock:
+                    assert await trader_inst.cancel_order(limit_buy) is True
+                    assert cancel_order_mock.call_count == 2
+                    _handle_order_cancel_error_mock.assert_called_once()
+                    _handle_order_cancel_error_mock.reset_mock()
+                    limit_buy.state = None
+
+        finally:
+            trader_inst.simulate = True
+
+
     async def test_cancel_open_orders_default_symbol(self):
         config, exchange_manager, trader_inst = await self.init_default()
         orders_manager = exchange_manager.exchange_personal_data.orders_manager
@@ -565,8 +634,9 @@ class TestTrader:
         with patch.object(ccxt.async_support.binanceus, "calculate_fee",
                           Mock(return_value=get_fees_mock("SOL"))) as calculate_fee_mock:
             await limit_buy.on_fill(force_fill=True)
-            # ensure call ccxt calculate_fee for order fees
-            calculate_fee_mock.assert_called_once()
+            # called twice: once for filled order fee calculation, once for forecasted fees calculation used to restore
+            # available amounts if relevant
+            assert calculate_fee_mock.call_count == 2
 
         # added filled orders as filled trades
         assert len(trades_manager.trades) == 1
@@ -609,7 +679,9 @@ class TestTrader:
         with patch.object(ccxt.async_support.binanceus, "calculate_fee", Mock(return_value=get_fees_mock("SOL"))) \
                 as calculate_fee_mock:
             await limit_buy.on_fill(force_fill=True)
-            calculate_fee_mock.assert_called_once()
+            # called twice: once for filled order fee calculation, once for forecasted fees calculation used to restore
+            # available amounts if relevant
+            assert calculate_fee_mock.call_count == 2
 
         assert limit_buy not in orders_manager.get_open_orders()
 
@@ -906,6 +978,7 @@ class TestTrader:
         )
         assert limit_buy.origin_quantity == decimal.Decimal("4")
         assert limit_buy.origin_price == decimal.Decimal("42")
+        assert limit_buy.filled_price == decimal.Decimal("42")  # simulated orders filled prices are synchronized
         assert limit_buy.origin_stop_price == decimal.Decimal("424")
         assert limit_buy.created_last_price == decimal.Decimal("4242")
         # also updated portfolio
@@ -1080,6 +1153,37 @@ async def test__has_open_position(future_trader_simulator_with_default_linear):
     assert trader_inst._has_open_position(DEFAULT_FUTURE_SYMBOL)
 
 
+async def test_create_order(future_trader_simulator_with_default_linear):
+    _, exchange_manager_inst, trader_inst, default_contract = future_trader_simulator_with_default_linear
+    order_mock = mock.Mock()
+    with mock.patch.object(trader_inst, "_create_new_order", mock.AsyncMock()) as _create_new_order_mock:
+        assert await trader_inst.create_order(order_mock)
+        _create_new_order_mock.assert_called_once_with(
+            order_mock, {}, True, constants.INDIVIDUAL_ORDER_SYNC_TIMEOUT
+        )
+    for err in (errors.MissingFunds, errors.AuthenticationError, errors.ExchangeCompliancyError):
+        with mock.patch.object(
+                trader_inst, "_create_new_order", mock.AsyncMock(side_effect=err)
+        ) as _create_new_order_mock:
+            with pytest.raises(err):
+                assert await trader_inst.create_order(
+                    order_mock, params="params", wait_for_creation=False, creation_timeout=1
+                )
+            _create_new_order_mock.assert_called_once_with(
+                order_mock, "params", False, 1
+            )
+        with mock.patch.object(
+            trader_inst, "_create_new_order", mock.AsyncMock(side_effect=ZeroDivisionError)
+        ) as _create_new_order_mock:
+            # does not raise
+            assert await trader_inst.create_order(
+                order_mock, params="params", wait_for_creation=False, creation_timeout=1
+            ) is None
+            _create_new_order_mock.assert_called_once_with(
+                order_mock, "params", False, 1
+            )
+
+
 def make_coroutine(response):
     async def coroutine(*args, **kwargs):
         return response
